# Copyright (C) 2009-2010 Red Hat, Inc.
#
# This program is free software; you can redistribute
# it and/or modify it under the terms of version 2 of the
# GNU General Public License as published by the
# Free Software Foundation.

# -*- coding: utf-8 -*-
"""The base Controller API."""

from tg import expose as _expose
from tg import TGController, request, tmpl_context

__all__ = ['_expose', 'BaseController', 'SubController']

class expose(_expose):
    """Own version of TG's 'expose' that forces using 'application/xhtml+xml'
    content type by default.
    """
    def __init__(self, template='', content_type=None,
                 exclude_names=None, custom_format=None, **kwargs):
        if content_type is None:
            if 'application/xhtml+xml' in request.accept:
                content_type = 'application/xhtml+xml; charset=utf-8'
            elif 'application/xml' in request.accept:
                content_type = 'application/xml; charset=utf-8'
            else:
                content_type = 'text/html; charset=utf-8'
        super(expose, self).__init__(template, content_type,
                                     exclude_names, custom_format, **kwargs)


class BaseController(TGController):
    """Base class for the root controller.

    The root of the application is used to compute URLs used by this app.

    """

    def __call__(self, environ, start_response):
        """Invoke the Controller."""
        # TGController.__call__ dispatches to the Controller method
        # the request is routed to. This routing information is
        # available in environ['pylons.routes_dict']

        request.identity = request.environ.get('repoze.who.identity')
        tmpl_context.identity = request.identity
        return TGController.__call__(self, environ, start_response)


class SubController(object):
    """Base class for subcontrollers.

    Keyword arguments passed on object creation are automatically moved
    inside the object as its attributes.

    """

    def __new__(cls, *args, **kwargs):
        # If the class that invoked this method is directly inherited from
        # this class, set new attributes according to passed keyword arguments
        # and delegate this invocation upwards (omitting the level
        # of SubController class).
        # Then delegate the invocation to superclass.

        if cls.__base__.__name__ == 'SubController':
            for kw, v in kwargs.iteritems():
                setattr(cls, kw, v)
            kwargs.clear()
        return super(type(cls), cls).__new__(cls, *args)
